const Sockpuppet = require('sockpuppet')
const _ = require('lodash')
const axios = require('axios')
const fs = require('fs')
const glob = require('glob')
const hourlyPercents = require('../configs/hourly-percents')
const moment = require('moment')
const path = require('path')
const userAgents = require('../configs/user-agents')

class Runner extends Sockpuppet {
  async preSetup() {
    // await this.setSlowMo()
    await this.setUserDataDir()
    this.setUserAgent()
  }

  async setSlowMo() {
    this.puppeteerOptions.slowMo = _.random(200, 300)
  }

  setUserAgent() {
    let userAgent = userAgents[_.random(0, userAgents.length - 1)]
    this.puppeteerOptions.args.push(`--user-agent='${userAgent}'`)
  }

  async setUserDataDir() {
    await this.setProxy()

    let userDataDir

    if (_.random(0, 100) <= -1) {
      userDataDir = this.createUserDataDir()
    } else {
      userDataDir = this.randomUserDataDir()
    }

    this.puppeteerOptions.userDataDir = userDataDir
  }

  async setProxy() {
    let info = { location: '', proxy: '' }
    let url = this.argv['proxy-url']

    if (typeof url === 'string') {
      await axios.get(url).then(resp => info.proxy = resp.data)
      await axios.get(`https://freeapi.ipip.net/${info.proxy.split(':')[0]}`)
        .then(resp => info.location = resp.data.join().split(',', 3).join(''))
    } else {
      await axios.get('https://myip.ipip.net/')
        .then(resp => info.location = resp.data.replace(/\s+/g, '').split('：')[2])
    }

    console.log(`proxy: ${info.proxy}\nlocation: ${info.location}`)
    this.puppeteerOptions.args.push(`--proxy-server=${info.proxy}`)

    this.proxyInfo = info
  }

  createUserDataDir() {
    console.log('new visitor')

    let task = this.argv._[0]
    let location = this.proxyInfo.location
    let date = moment().format('YYYYMMDD')
    let time = moment().format('HHmmss')

    return path.join(process.cwd(), 'states', task, location, date, time)
  }

  randomUserDataDir() {
    let task = this.argv._[0]
    let location = this.proxyInfo.location
    let date
    let userDataDir

    if (_.random(0, 100) <= 900) {
      let dateSeeder = _.random(0, 100)

      if (dateSeeder <= 500) {
        date = moment().format('YYYYMMDD')
      } else if (dateSeeder <= 80) {
        date = moment().subtract(1, 'days').format('YYYYMMDD')
      } else {
        date = moment().subtract(2, 'days').format('YYYYMMDD')
      }

      let pattern = `states/${task}/${location}/${date}/*`
      let userDataDirs = glob.sync(pattern)

      console.log(`${pattern}: ${userDataDirs.length}`)

      if (userDataDirs.length > 0) {
        console.log('old visitor')
        userDataDir = userDataDirs[_.random(0, userDataDirs.length - 1)]
      } else {
        userDataDir = this.createUserDataDir()
      }
    } else {
      userDataDir = this.createUserDataDir()
    }

    return userDataDir
  }

  /**
   * duration: average seconds
   * daily-quota: max times
   */
  async postTeardown() {
    let duration = this.argv.duration || 60
    let dailyQuota = this.argv['daily-quota'] || 500
    let hourlyQuota = _.ceil(dailyQuota * (hourlyPercents[moment().hour()] / 100))
    console.log(`max ${hourlyQuota} times in current hour`)
    let delay = _.ceil((60 * 60 * 1000) / (hourlyQuota) - duration * 1000)
    delay = _.random(0, delay * 2)
    delay = _.max([0, delay])
    console.log(`session ended, delay ${delay / 1000}s\n`)
    setTimeout(() => this.start(), delay);
  }
}

module.exports = Runner
